@@ -21,6 +21,8 @@
 
 #include "kate_ctags_plugin.h"
 
+#include "hostprocess.h"
+
 #include <QCheckBox>
 #include <QFileDialog>
 #include <QFileInfo>
@@ -213,7 +215,7 @@ void KateCTagsConfigPage::updateGlobalDB()
         targets += QLatin1Char('\"') + target + QLatin1String("\" ");
     }
 
-    QString file = QStandardPaths::writableLocation(QStandardPaths::DataLocation) + QLatin1String("/katectags");
+    QString file = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + QLatin1String("/katectags");
     QDir().mkpath(file);
     file += QLatin1String("/common_db");
 
@@ -225,7 +227,7 @@ void KateCTagsConfigPage::updateGlobalDB()
     QString commandLine = QStringLiteral("%1 -f %2 %3").arg(m_confUi.cmdEdit->text(), file, targets);
     QStringList arguments = m_proc.splitCommand(commandLine);
     QString command = arguments.takeFirst();
-    m_proc.start(command, arguments);
+    startHostProcess(m_proc, command, arguments);
 
     if (!m_proc.waitForStarted(500)) {
         KMessageBox::error(nullptr, i18n("Failed to run \"%1\". exitStatus = %2", commandLine, m_proc.exitStatus()));
