@@ -49,13 +49,13 @@ KateSQLView::KateSQLView(KTextEditor::Plugin *plugin, KTextEditor::MainWindow *m
     m_outputToolView = mw->createToolView(plugin,
                                           QStringLiteral("kate_private_plugin_katesql_output"),
                                           KTextEditor::MainWindow::Bottom,
-                                          QIcon::fromTheme(QStringLiteral("view-form-table")),
+                                          QIcon::fromTheme(QStringLiteral("server-database")),
                                           i18nc("@title:window", "SQL"));
 
     m_schemaBrowserToolView = mw->createToolView(plugin,
                                                  QStringLiteral("kate_private_plugin_katesql_schemabrowser"),
                                                  KTextEditor::MainWindow::Left,
-                                                 QIcon::fromTheme(QStringLiteral("view-list-tree")),
+                                                 QIcon::fromTheme(QStringLiteral("server-database")),
                                                  i18nc("@title:window", "SQL Schema"));
 
     m_outputWidget = new KateSQLOutputWidget(m_outputToolView);
@@ -131,7 +131,6 @@ void KateSQLView::setupActions()
     action = collection->addAction(QStringLiteral("query_run"));
     action->setText(i18nc("@action:inmenu", "Run query"));
     action->setIcon(QIcon::fromTheme(QStringLiteral("quickopen")));
-    collection->setDefaultShortcut(action, QKeySequence(Qt::CTRL | Qt::Key_E));
     connect(action, &QAction::triggered, this, &KateSQLView::slotRunQuery);
 
     /// TODO: stop sql query
