@@ -40,13 +40,12 @@
 #include <KAuthorized>
 #include <KConfigGroup>
 #include <KPluginFactory>
-#include <KPluginLoader>
 #include <KSharedConfig>
 #include <KXMLGUIFactory>
 
 K_PLUGIN_FACTORY_WITH_JSON(KateKonsolePluginFactory, "katekonsoleplugin.json", registerPlugin<KateKonsolePlugin>();)
 
-static const QStringList s_escapeExceptions{QStringLiteral("vi"), QStringLiteral("vim"), QStringLiteral("nvim")};
+static const QStringList s_escapeExceptions{QStringLiteral("vi"), QStringLiteral("vim"), QStringLiteral("nvim"), QStringLiteral("git")};
 
 KateKonsolePlugin::KateKonsolePlugin(QObject *parent, const QList<QVariant> &)
     : KTextEditor::Plugin(parent)
@@ -196,12 +195,12 @@ void KateConsole::loadConsoleIfNeeded()
     /**
      * get konsole part factory
      */
-    KPluginFactory *factory = KPluginLoader(QStringLiteral("konsolepart")).factory();
+    KPluginFactory *factory = KPluginFactory::loadFactory(QStringLiteral("konsolepart")).plugin;
     if (!factory) {
         return;
     }
 
-    m_part = factory->create<KParts::ReadOnlyPart>(this, this);
+    m_part = factory->create<KParts::ReadOnlyPart>(this);
 
     if (!m_part) {
         return;
@@ -607,7 +606,7 @@ QString KateKonsoleConfigPage::fullName() const
 
 QIcon KateKonsoleConfigPage::icon() const
 {
-    return QIcon::fromTheme(QStringLiteral("utilities-terminal"));
+    return QIcon::fromTheme(QStringLiteral("dialog-scripts"));
 }
 
 void KateKonsoleConfigPage::apply()
