@@ -30,18 +30,19 @@
 #include <QHeaderView>
 #include <QInputMethodEvent>
 #include <QLineEdit>
+#include <QMap>
 #include <QMenu>
 #include <QPainter>
 #include <QPlainTextEdit>
 #include <QPushButton>
 #include <QStringListModel>
 #include <QStyledItemDelegate>
-#include <QTimer>
 #include <QToolButton>
 #include <QTreeView>
 #include <QVBoxLayout>
 #include <QtConcurrentRun>
 
+#include <KActionCollection>
 #include <KLocalizedString>
 #include <KMessageBox>
 
@@ -179,18 +180,24 @@ public:
     }
 };
 
-static QToolButton *toolButton(const QString &icon, const QString &tooltip, const QString &text = QString(), Qt::ToolButtonStyle t = Qt::ToolButtonIconOnly)
+static QToolButton *toolButton(Qt::ToolButtonStyle t = Qt::ToolButtonIconOnly)
 {
     auto tb = new QToolButton;
-    tb->setIcon(QIcon::fromTheme(icon));
-    tb->setToolTip(tooltip);
-    tb->setText(text);
     tb->setAutoRaise(true);
     tb->setToolButtonStyle(t);
     tb->setSizePolicy(QSizePolicy::Minimum, tb->sizePolicy().verticalPolicy());
     return tb;
 }
 
+static QToolButton *toolButton(const QString &icon, const QString &tooltip, const QString &text = QString(), Qt::ToolButtonStyle t = Qt::ToolButtonIconOnly)
+{
+    auto tb = toolButton(t);
+    tb->setToolTip(tooltip);
+    tb->setIcon(QIcon::fromTheme(icon));
+    tb->setText(text);
+    return tb;
+}
+
 GitWidget::GitWidget(KateProject *project, KTextEditor::MainWindow *mainWindow, KateProjectPluginView *pluginView)
     : m_project(project)
     , m_mainWin(mainWindow)
@@ -201,8 +208,9 @@ GitWidget::GitWidget(KateProject *project, KTextEditor::MainWindow *mainWindow,
     setDotGitPath();
 
     m_treeView = new GitWidgetTreeView(this);
+    auto ac = m_pluginView->actionCollection();
 
-    buildMenu();
+    buildMenu(ac);
     m_menuBtn = toolButton(QStringLiteral("application-menu"), QString());
     m_menuBtn->setMenu(m_gitMenu);
     m_menuBtn->setArrowType(Qt::NoArrow);
@@ -211,22 +219,37 @@ GitWidget::GitWidget(KateProject *project, KTextEditor::MainWindow *mainWindow,
         m_menuBtn->showMenu();
     });
 
-    m_commitBtn = toolButton(QStringLiteral("vcs-commit"), QString(), i18n("Commit"), Qt::ToolButtonTextBesideIcon);
+    const QString &commitText = i18n("Commit");
+    const QIcon &commitIcon = QIcon::fromTheme(QStringLiteral("vcs-commit"));
+    m_commitBtn = toolButton(Qt::ToolButtonTextBesideIcon);
+    m_commitBtn->setIcon(commitIcon);
+    m_commitBtn->setText(commitText);
+    m_commitBtn->setToolTip(commitText);
     m_commitBtn->setMinimumHeight(16);
 
-    m_pushBtn = toolButton(QStringLiteral("vcs-push"), i18n("Git push"));
-    connect(m_pushBtn, &QToolButton::clicked, this, [this]() {
+    const QString &pushText = i18n("Git push");
+    m_pushBtn = toolButton();
+    auto a = ac->addAction(QStringLiteral("vcs_push"), this, [this]() {
         PushPullDialog ppd(m_mainWin, m_gitPath);
         connect(&ppd, &PushPullDialog::runGitCommand, this, &GitWidget::runPushPullCmd);
         ppd.openDialog(PushPullDialog::Push);
     });
+    a->setIcon(QIcon::fromTheme(QStringLiteral("vcs-push")));
+    a->setText(pushText);
+    a->setToolTip(pushText);
+    m_pushBtn->setDefaultAction(a);
 
-    m_pullBtn = toolButton(QStringLiteral("vcs-pull"), i18n("Git pull"));
-    connect(m_pullBtn, &QToolButton::clicked, this, [this]() {
+    const QString &pullText = i18n("Git pull");
+    m_pullBtn = toolButton(QStringLiteral("vcs-pull"), pullText);
+    a = ac->addAction(QStringLiteral("vcs_pull"), this, [this]() {
         PushPullDialog ppd(m_mainWin, m_gitPath);
         connect(&ppd, &PushPullDialog::runGitCommand, this, &GitWidget::runPushPullCmd);
         ppd.openDialog(PushPullDialog::Pull);
     });
+    a->setIcon(QIcon::fromTheme(QStringLiteral("vcs-pull")));
+    a->setText(pullText);
+    a->setToolTip(pullText);
+    m_pullBtn->setDefaultAction(a);
 
     m_cancelBtn = toolButton(QStringLiteral("dialog-cancel"), i18n("Cancel Operation"));
     m_cancelBtn->setHidden(true);
@@ -257,7 +280,7 @@ GitWidget::GitWidget(KateProject *project, KTextEditor::MainWindow *mainWindow,
     layout->addWidget(m_treeView);
 
     m_filterLineEdit = new QLineEdit(this);
-    m_filterLineEdit->setPlaceholderText(i18n("Type to filter..."));
+    m_filterLineEdit->setPlaceholderText(i18n("Filter..."));
     layout->addWidget(m_filterLineEdit);
 
     m_model = new GitStatusModel(this);
@@ -274,6 +297,7 @@ GitWidget::GitWidget(KateProject *project, KTextEditor::MainWindow *mainWindow,
     m_treeView->installEventFilter(this);
     m_treeView->setRootIsDecorated(false);
     m_treeView->setAllColumnsShowFocus(true);
+    m_treeView->expandAll();
 
     if (m_treeView->style()) {
         auto indent = m_treeView->style()->pixelMetric(QStyle::PM_TreeViewIndentation, nullptr, m_treeView);
@@ -288,8 +312,18 @@ GitWidget::GitWidget(KateProject *project, KTextEditor::MainWindow *mainWindow,
     // our main view - status view + btns
     m_mainView->setLayout(layout);
 
+    a = ac->addAction(QStringLiteral("vcs_commit"), this, [this] {
+        openCommitChangesDialog();
+        slotUpdateStatus();
+    });
+    a->setText(commitText);
+    a->setToolTip(commitText);
+    a->setIcon(commitIcon);
+
     connect(&m_gitStatusWatcher, &QFutureWatcher<GitUtils::GitParsedStatus>::finished, this, &GitWidget::parseStatusReady);
     connect(m_commitBtn, &QPushButton::clicked, this, &GitWidget::openCommitChangesDialog);
+    // This may not needed anylonger, but we do it anyway, just to be on the save side, see e239cb310
+    connect(m_commitBtn, &QPushButton::pressed, this, &GitWidget::slotUpdateStatus);
 
     // single / double click
     connect(m_treeView, &QTreeView::clicked, this, &GitWidget::treeViewSingleClicked);
@@ -301,6 +335,11 @@ GitWidget::GitWidget(KateProject *project, KTextEditor::MainWindow *mainWindow,
     setLayout(new QVBoxLayout);
     this->layout()->addWidget(m_stackWidget);
     this->layout()->setContentsMargins(0, 0, 0, 0);
+
+    // Setup update protection
+    m_updateTrigger.setSingleShot(true);
+    m_updateTrigger.setInterval(500);
+    connect(&m_updateTrigger, &QTimer::timeout, this, &GitWidget::slotUpdateStatus);
 }
 
 GitWidget::~GitWidget()
@@ -323,7 +362,7 @@ void GitWidget::setDotGitPath()
     const auto dotGitPath = GitUtils::getDotGitPath(m_project->baseDir());
     if (!dotGitPath.has_value()) {
         QTimer::singleShot(1, this, [this] {
-            sendMessage(i18n("Failed to find .git directory, things may not work correctly"), false);
+            sendMessage(i18n("Failed to find .git directory for '%1', things may not work correctly", m_project->baseDir()), false);
         });
         m_gitPath = m_project->baseDir();
         return;
@@ -360,18 +399,19 @@ QProcess *GitWidget::gitp(const QStringList &arguments)
     return git;
 }
 
-void GitWidget::getStatus(bool untracked, bool submodules)
+void GitWidget::updateStatus()
 {
-    auto args = QStringList{QStringLiteral("status"), QStringLiteral("-z")};
-    if (!untracked) {
-        args.append(QStringLiteral("-uno"));
-    } else {
-        args.append(QStringLiteral("-u"));
-    }
-    if (!submodules) {
-        args.append(QStringLiteral("--ignore-submodules"));
+    m_updateTrigger.start();
+}
+
+void GitWidget::slotUpdateStatus()
+{
+    if (!isVisible()) {
+        return; // No need to update
     }
 
+    const auto args = QStringList{QStringLiteral("status"), QStringLiteral("-z"), QStringLiteral("-u"), QStringLiteral("--ignore-submodules")};
+
     auto git = gitp(args);
     connect(git, &QProcess::finished, this, [this, git](int exitCode, QProcess::ExitStatus es) {
         if (es != QProcess::NormalExit || exitCode != 0) {
@@ -384,7 +424,7 @@ void GitWidget::getStatus(bool untracked, bool submodules)
         }
         git->deleteLater();
     });
-    git->start(QProcess::ReadOnly);
+    startHostProcess(*git, QProcess::ReadOnly);
 }
 
 void GitWidget::runGitCmd(const QStringList &args, const QString &i18error)
@@ -394,11 +434,11 @@ void GitWidget::runGitCmd(const QStringList &args, const QString &i18error)
         if (es != QProcess::NormalExit || exitCode != 0) {
             sendMessage(i18error + QStringLiteral(": ") + QString::fromUtf8(git->readAllStandardError()), true);
         } else {
-            getStatus();
+            updateStatus();
         }
         git->deleteLater();
     });
-    git->start(QProcess::ReadOnly);
+    startHostProcess(*git, QProcess::ReadOnly);
 }
 
 void GitWidget::runPushPullCmd(const QStringList &args)
@@ -415,14 +455,14 @@ void GitWidget::runPushPullCmd(const QStringList &args)
             QString cmd = gargs.join(QStringLiteral(" "));
             QString out = QString::fromUtf8(git->readAll());
             sendMessage(i18n("\"%1\" executed successfully: %2", cmd, out), false);
-            getStatus();
+            updateStatus();
         }
         hideCancel();
         git->deleteLater();
     });
 
     enableCancel(git);
-    git->start(QProcess::ReadOnly);
+    startHostProcess(*git, QProcess::ReadOnly);
 }
 
 void GitWidget::stage(const QStringList &files, bool)
@@ -481,7 +521,7 @@ void GitWidget::openAtHEAD(const QString &file)
     auto args = QStringList{QStringLiteral("show"), QStringLiteral("--textconv")};
     args.append(QStringLiteral(":") + file);
     auto git = gitp(args);
-    git->start(QProcess::ReadOnly);
+    startHostProcess(*git, QProcess::ReadOnly);
 
     connect(git, &QProcess::finished, this, [this, file, git](int exitCode, QProcess::ExitStatus es) {
         if (es != QProcess::NormalExit || exitCode != 0) {
@@ -499,7 +539,7 @@ void GitWidget::openAtHEAD(const QString &file)
     });
 
     git->setArguments(args);
-    git->start(QProcess::ReadOnly);
+    startHostProcess(*git, QProcess::ReadOnly);
 }
 
 void GitWidget::showDiff(const QString &file, bool staged)
@@ -554,7 +594,7 @@ void GitWidget::showDiff(const QString &file, bool staged)
         }
         git->deleteLater();
     });
-    git->start(QProcess::ReadOnly);
+    startHostProcess(*git, QProcess::ReadOnly);
 }
 
 void GitWidget::launchExternalDiffTool(const QString &file, bool staged)
@@ -601,12 +641,12 @@ void GitWidget::commitChanges(const QString &msg, const QString &desc, bool sign
             sendMessage(i18n("Failed to commit: %1", QString::fromUtf8(git->readAllStandardError())), true);
         } else {
             m_commitMessage.clear();
-            getStatus();
+            updateStatus();
             sendMessage(i18n("Changes committed successfully."), false);
         }
         git->deleteLater();
     });
-    git->start(QProcess::ReadOnly);
+    startHostProcess(*git, QProcess::ReadOnly);
 }
 
 QString GitWidget::getDiff(KTextEditor::View *v, bool hunk, bool alreadyStaged)
@@ -659,13 +699,13 @@ void GitWidget::applyDiff(const QString &fileName, bool staged, bool hunk, KText
             }
             // must come at the end
             QTimer::singleShot(10, this, [this] {
-                getStatus();
+                updateStatus();
             });
         }
         delete file;
         git->deleteLater();
     });
-    git->start(QProcess::ReadOnly);
+    startHostProcess(*git, QProcess::ReadOnly);
 }
 
 void GitWidget::openCommitChangesDialog(bool amend)
@@ -716,7 +756,7 @@ void GitWidget::handleClick(const QModelIndex &idx, ClickAction clickAction)
         return stage({file});
     }
 
-    if (clickAction == ClickAction::ShowDiff) {
+    if (clickAction == ClickAction::ShowDiff && statusItemType != GitStatusModel::NodeUntrack) {
         showDiff(file, staged);
     }
 
@@ -735,35 +775,45 @@ void GitWidget::treeViewDoubleClicked(const QModelIndex &idx)
     handleClick(idx, m_pluginView->plugin()->doubleClickAcion());
 }
 
-void GitWidget::hideEmptyTreeNodes()
+void GitWidget::parseStatusReady()
 {
-    auto expand = [this](GitStatusModel::ItemType t) {
-        auto *model = m_treeView->model();
-        auto index = model->index(t, 0);
-        if (!index.isValid() || index.data(GitStatusModel::TreeItemType).toInt() == GitStatusModel::NodeUntrack) {
-            return;
-        }
-        if (model->rowCount(index) > 0 && !m_treeView->isExpanded(index)) {
-            m_treeView->expand(index);
-        }
-    };
+    // Remember collapse/expand state
+    // The default is expanded, so only add here which should be not expanded
+    QMap<int, bool> nodeIsExpanded;
+    nodeIsExpanded.insert(GitStatusModel::NodeUntrack, false);
 
-    expand(GitStatusModel::NodeStage);
-    expand(GitStatusModel::NodeChanges);
-    expand(GitStatusModel::NodeConflict);
+    const auto *model = m_treeView->model();
+    for (int i = 0; i < model->rowCount(); ++i) {
+        const auto index = model->index(i, 0);
+        if (!index.isValid()) {
+            continue;
+        }
+        const auto t = index.data(GitStatusModel::TreeItemType).toInt();
+        nodeIsExpanded[t] = m_treeView->isExpanded(index);
+    }
+
+    // Set new data
+    GitUtils::GitParsedStatus s = m_gitStatusWatcher.result();
+    m_model->setStatusItems(std::move(s), m_pluginView->plugin()->showGitStatusWithNumStat());
+
+    // Restore collapse/expand state
+    for (int i = 0; i < model->rowCount(); ++i) {
+        const auto index = model->index(i, 0);
+        if (!index.isValid()) {
+            continue;
+        }
+        const auto t = index.data(GitStatusModel::TreeItemType).toInt();
+        if (!nodeIsExpanded.contains(t) || nodeIsExpanded[t]) {
+            m_treeView->expand(index);
+        } else {
+            m_treeView->collapse(index);
+        }
+    }
 
     m_treeView->resizeColumnToContents(0);
     m_treeView->resizeColumnToContents(1);
 }
 
-void GitWidget::parseStatusReady()
-{
-    GitUtils::GitParsedStatus s = m_gitStatusWatcher.result();
-
-    m_model->setStatusItems(std::move(s), m_pluginView->plugin()->showGitStatusWithNumStat());
-    hideEmptyTreeNodes();
-}
-
 void GitWidget::branchCompareFiles(const QString &from, const QString &to)
 {
     if (from.isEmpty() && to.isEmpty()) {
@@ -780,7 +830,7 @@ void GitWidget::branchCompareFiles(const QString &from, const QString &to)
         return;
     }
 
-    git.start(QProcess::ReadOnly);
+    startHostProcess(git, QProcess::ReadOnly);
     if (git.waitForStarted() && git.waitForFinished(-1)) {
         if (git.exitStatus() != QProcess::NormalExit || git.exitCode() != 0) {
             return;
@@ -807,7 +857,7 @@ void GitWidget::branchCompareFiles(const QString &from, const QString &to)
         return;
     }
 
-    git.start(QProcess::ReadOnly);
+    startHostProcess(git, QProcess::ReadOnly);
     if (git.waitForStarted() && git.waitForFinished(-1)) {
         if (git.exitStatus() != QProcess::NormalExit || git.exitCode() != 0) {
             sendMessage(i18n("Failed to get numstat when diffing %1...%2", from, to), true);
@@ -841,36 +891,45 @@ bool GitWidget::eventFilter(QObject *o, QEvent *e)
     return QWidget::eventFilter(o, e);
 }
 
-void GitWidget::buildMenu()
+void GitWidget::buildMenu(KActionCollection *ac)
 {
     m_gitMenu = new QMenu(this);
-    auto r = m_gitMenu->addAction(i18n("Refresh"), this, [this] {
+    auto a = ac->addAction(QStringLiteral("vcs_status_refresh"), this, [this] {
         if (m_project) {
-            getStatus();
+            updateStatus();
         }
     });
-    r->setIcon(QIcon::fromTheme(QStringLiteral("view-refresh")));
+    a->setText(i18n("Refresh"));
+    a->setIcon(QIcon::fromTheme(QStringLiteral("view-refresh")));
+    m_gitMenu->addAction(a);
 
-    m_gitMenu->addAction(QIcon::fromTheme(QStringLiteral("document-edit")), i18n("Amend Last Commit"), this, [this] {
+    a = ac->addAction(QStringLiteral("vcs_amend"), this, [this] {
         openCommitChangesDialog(/* amend = */ true);
     });
+    a->setIcon(QIcon::fromTheme(QStringLiteral("document-edit")));
+    a->setText(i18n("Amend Last Commit"));
+    m_gitMenu->addAction(a);
 
-    auto a = m_gitMenu->addAction(i18n("Checkout Branch"), this, [this] {
+    a = ac->addAction(QStringLiteral("vcs_branch_checkout"), this, [this] {
         BranchCheckoutDialog bd(m_mainWin->window(), m_pluginView, m_project->baseDir());
         bd.openDialog();
     });
+    a->setText(i18n("Checkout Branch"));
     a->setIcon(QIcon::fromTheme(QStringLiteral("vcs-branch")));
+    m_gitMenu->addAction(a);
 
-    a = m_gitMenu->addAction(i18n("Delete Branch"), this, [this] {
+    a = ac->addAction(QStringLiteral("vcs_branch_delete"), this, [this] {
         BranchDeleteDialog dlg(m_gitPath, this);
         if (dlg.exec() == QDialog::Accepted) {
             auto result = GitUtils::deleteBranches(dlg.branchesToDelete(), m_gitPath);
             sendMessage(result.error, result.returnCode != 0);
         }
     });
+    a->setText(i18n("Delete Branch"));
     a->setIcon(QIcon::fromTheme(QStringLiteral("edit-delete")));
+    m_gitMenu->addAction(a);
 
-    a = m_gitMenu->addAction(i18n("Compare Branch with ..."), this, [this] {
+    a = ac->addAction(QStringLiteral("vcs_branch_diff"), this, [this] {
         BranchesDialog bd(m_mainWin->window(), m_pluginView, m_project->baseDir());
         using GitUtils::RefType;
         bd.openDialog(static_cast<GitUtils::RefType>(RefType::Head | RefType::Remote));
@@ -878,9 +937,11 @@ void GitWidget::buildMenu()
         branchCompareFiles(branch, QString());
     });
     a->setIcon(QIcon::fromTheme(QStringLiteral("vcs-diff")));
+    a->setText(i18n("Compare Branch with ..."));
+    m_gitMenu->addAction(a);
 
     auto stashMenu = m_gitMenu->addAction(QIcon::fromTheme(QStringLiteral("vcs-stash")), i18n("Stash"));
-    stashMenu->setMenu(this->stashMenu());
+    stashMenu->setMenu(this->stashMenu(ac));
 }
 
 void GitWidget::createStashDialog(StashMode m, const QString &gitPath)
@@ -891,7 +952,7 @@ void GitWidget::createStashDialog(StashMode m, const QString &gitPath)
         m_pluginView->showDiffInFixedView(r);
     });
     connect(stashDialog, &StashDialog::done, this, [this, stashDialog] {
-        getStatus();
+        updateStatus();
         stashDialog->deleteLater();
     });
     stashDialog->openDialog(m);
@@ -910,53 +971,50 @@ void GitWidget::hideCancel()
     m_pushBtn->show();
 }
 
-QMenu *GitWidget::stashMenu()
+QMenu *GitWidget::stashMenu(KActionCollection *ac)
 {
     QMenu *menu = new QMenu(this);
-    auto stashAct = menu->addAction(QIcon::fromTheme(QStringLiteral("vcs-stash")), i18n("Stash"));
-    auto popLastAct = menu->addAction(QIcon::fromTheme(QStringLiteral("vcs-stash-pop")), i18n("Pop Last Stash"));
-    auto popAct = menu->addAction(QIcon::fromTheme(QStringLiteral("vcs-stash-pop")), i18n("Pop Stash"));
-    auto applyLastAct = menu->addAction(i18n("Apply Last Stash"));
-    auto stashKeepStagedAct = menu->addAction(QIcon::fromTheme(QStringLiteral("vcs-stash")), i18n("Stash (Keep Staged)"));
-    auto stashUAct = menu->addAction(QIcon::fromTheme(QStringLiteral("vcs-stash")), i18n("Stash (Include Untracked)"));
-    auto applyStashAct = menu->addAction(i18n("Apply Stash"));
-    auto dropAct = menu->addAction(i18n("Drop Stash"));
-    auto showStashAct = menu->addAction(i18n("Show Stash Content"));
+    auto a = stashMenuAction(ac, QStringLiteral("vcs_stash"), i18n("Stash"), StashMode::Stash);
+    a->setIcon(QIcon::fromTheme(QStringLiteral("vcs-stash")));
+    menu->addAction(a);
 
-    connect(stashAct, &QAction::triggered, this, [this] {
-        createStashDialog(StashMode::Stash, m_gitPath);
-    });
-    connect(stashUAct, &QAction::triggered, this, [this] {
-        createStashDialog(StashMode::StashUntrackIncluded, m_gitPath);
-    });
-    connect(stashKeepStagedAct, &QAction::triggered, this, [this] {
-        createStashDialog(StashMode::StashKeepIndex, m_gitPath);
-    });
-    connect(popAct, &QAction::triggered, this, [this] {
-        createStashDialog(StashMode::StashPop, m_gitPath);
-    });
-    connect(applyStashAct, &QAction::triggered, this, [this] {
-        createStashDialog(StashMode::StashApply, m_gitPath);
-    });
-    connect(dropAct, &QAction::triggered, this, [this] {
-        createStashDialog(StashMode::StashDrop, m_gitPath);
-    });
-    connect(popLastAct, &QAction::triggered, this, [this] {
-        createStashDialog(StashMode::StashPopLast, m_gitPath);
-    });
-    connect(applyLastAct, &QAction::triggered, this, [this] {
-        createStashDialog(StashMode::StashApplyLast, m_gitPath);
-    });
-    connect(showStashAct, &QAction::triggered, this, [this] {
-        createStashDialog(StashMode::ShowStashContent, m_gitPath);
-    });
+    a = stashMenuAction(ac, QStringLiteral("vcs_stash_pop_last"), i18n("Pop Last Stash"), StashMode::StashPopLast);
+    a->setIcon(QIcon::fromTheme(QStringLiteral("vcs-stash-pop")));
+    menu->addAction(a);
+
+    a = stashMenuAction(ac, QStringLiteral("vcs_stash_pop"), i18n("Pop Stash"), StashMode::StashPop);
+    a->setIcon(QIcon::fromTheme(QStringLiteral("vcs-stash-pop")));
+    menu->addAction(a);
+
+    menu->addAction(stashMenuAction(ac, QStringLiteral("vcs_stash_apply_last"), i18n("Apply Last Stash"), StashMode::StashApplyLast));
+
+    a = stashMenuAction(ac, QStringLiteral("vcs_stash_keep_staged"), i18n("Stash (Keep Staged)"), StashMode::StashKeepIndex);
+    a->setIcon(QIcon::fromTheme(QStringLiteral("vcs-stash")));
+    menu->addAction(a);
+
+    a = stashMenuAction(ac, QStringLiteral("vcs_stash_include_untracked"), i18n("Stash (Include Untracked)"), StashMode::StashUntrackIncluded);
+    a->setIcon(QIcon::fromTheme(QStringLiteral("vcs-stash")));
+    menu->addAction(a);
+
+    menu->addAction(stashMenuAction(ac, QStringLiteral("vcs_stash_apply"), i18n("Apply Stash"), StashMode::StashApply));
+    menu->addAction(stashMenuAction(ac, QStringLiteral("vcs_stash_drop"), i18n("Drop Stash"), StashMode::StashDrop));
+    menu->addAction(stashMenuAction(ac, QStringLiteral("vcs_stash_show"), i18n("Show Stash Content"), StashMode::ShowStashContent));
 
     return menu;
 }
 
-static KMessageBox::ButtonCode confirm(GitWidget *_this, const QString &text)
+QAction *GitWidget::stashMenuAction(KActionCollection *ac, const QString &name, const QString &text, StashMode m)
 {
-    return KMessageBox::questionYesNo(_this, text, {}, KStandardGuiItem::yes(), KStandardGuiItem::no(), {}, KMessageBox::Dangerous);
+    auto a = ac->addAction(name, this, [this, m] {
+        createStashDialog(m, m_gitPath);
+    });
+    a->setText(text);
+    return a;
+}
+
+static KMessageBox::ButtonCode confirm(GitWidget *_this, const QString &text, const KGuiItem &confirmItem)
+{
+    return KMessageBox::questionYesNo(_this, text, {}, confirmItem, KStandardGuiItem::cancel(), {}, KMessageBox::Dangerous);
 }
 
 void GitWidget::treeViewContextMenuEvent(QContextMenuEvent *e)
@@ -997,12 +1055,12 @@ void GitWidget::treeViewContextMenuEvent(QContextMenuEvent *e)
         if (act == stageAct) {
             stage(files, treeItem == GitStatusModel::NodeUntrack);
         } else if (act == discardAct && !untracked) {
-            auto ret = confirm(this, i18n("Are you sure you want to remove these files?"));
+            auto ret = confirm(this, i18n("Are you sure you want to remove these files?"), KStandardGuiItem::remove());
             if (ret == KMessageBox::Yes) {
                 discard(files);
             }
         } else if (act == discardAct && untracked) {
-            auto ret = confirm(this, i18n("Are you sure you want to discard all changes?"));
+            auto ret = confirm(this, i18n("Are you sure you want to discard all changes?"), KStandardGuiItem::discard());
             if (ret == KMessageBox::Yes) {
                 clean(files);
             }
@@ -1048,7 +1106,7 @@ void GitWidget::treeViewContextMenuEvent(QContextMenuEvent *e)
             }
             return stage({file});
         } else if (act == discardAct && !untracked) {
-            auto ret = confirm(this, i18n("Are you sure you want to discard the changes in this file?"));
+            auto ret = confirm(this, i18n("Are you sure you want to discard the changes in this file?"), KStandardGuiItem::discard());
             if (ret == KMessageBox::Yes) {
                 discard({file});
             }
@@ -1057,7 +1115,7 @@ void GitWidget::treeViewContextMenuEvent(QContextMenuEvent *e)
         } else if (showDiffAct && act == showDiffAct && !untracked) {
             showDiff(file, staged);
         } else if (act == discardAct && untracked) {
-            auto ret = confirm(this, i18n("Are you sure you want to remove this file?"));
+            auto ret = confirm(this, i18n("Are you sure you want to remove this file?"), KStandardGuiItem::remove());
             if (ret == KMessageBox::Yes) {
                 clean({file});
             }
@@ -1148,12 +1206,12 @@ void GitWidget::selectedContextMenu(QContextMenuEvent *e)
             unstage(files);
         }
     } else if (selectionHasChangedItems && !selectionHasUntrackedItems && execAct == discardAct) {
-        auto ret = confirm(this, i18n("Are you sure you want to discard the changes?"));
+        auto ret = confirm(this, i18n("Are you sure you want to discard the changes?"), KStandardGuiItem::discard());
         if (ret == KMessageBox::Yes) {
             discard(files);
         }
     } else if (!selectionHasChangedItems && selectionHasUntrackedItems && execAct == removeAct) {
-        auto ret = confirm(this, i18n("Are you sure you want to remove these untracked changes?"));
+        auto ret = confirm(this, i18n("Are you sure you want to remove these untracked changes?"), KStandardGuiItem::remove());
         if (ret == KMessageBox::Yes) {
             clean(files);
         }
