@@ -12,6 +12,7 @@
 #include <QMenu>
 #include <QPointer>
 #include <QStackedWidget>
+#include <QTimer>
 #include <QToolButton>
 
 #include <KTextEditor/View>
@@ -19,10 +20,11 @@
 
 #include <memory>
 
+#include <kateprojectview.h>
+
 class QAction;
 class QDir;
 class KateProject;
-class KateProjectView;
 class KateProjectPlugin;
 class KateProjectInfoView;
 
@@ -191,15 +193,24 @@ private Q_SLOTS:
     void slotProjectReload();
 
     /**
-     * Getting project for others windows
-     * and closing project documents.
+     * Close currently active project.
      */
-    void slotProjectAboutToClose();
+    void slotCloseProject();
 
     /**
-     * Close current project.
+     * Close all projects.
      */
-    void slotProjectClose(KateProject *project);
+    void slotCloseAllProjects();
+
+    /**
+     * Close all projects without open documents.
+     */
+    void slotCloseAllProjectsWithoutDocuments();
+
+    /**
+     * Handle closing of a project.
+     */
+    void slotHandleProjectClosing(KateProject *project);
 
     /**
      * Lookup current word
@@ -287,6 +298,11 @@ private Q_SLOTS:
      */
     void slotDocumentUrlChanged(KTextEditor::Document *document);
 
+    /**
+     * A helper to trigger an update of the git-widget.
+     */
+    void slotDocumentSaved();
+
     /**
      * Show context menu
      */
@@ -324,6 +340,11 @@ private:
     QString currentWord() const;
 
 private:
+    /**
+     * Watches for changes to .git/indiex
+     */
+    QFileSystemWatcher m_gitChangedWatcher;
+
     /**
      * our plugin
      */
@@ -370,7 +391,7 @@ private:
     QToolButton *m_reloadButton;
 
     /**
-     * Closeing button
+     * Closing button for current project
      */
     QToolButton *m_closeProjectButton;
 
@@ -392,7 +413,7 @@ private:
     /**
      * stacked widget will all currently created git views
      */
-    QStackedWidget *m_stackedgitViews;
+    QStackedWidget *m_stackedGitViews;
 
     /**
      * project => view
@@ -421,6 +442,8 @@ private:
     QAction *m_projectNextAction;
     QAction *m_projectGotoIndexAction;
     QAction *m_projectCloseAction;
+    QAction *m_projectCloseAllAction;
+    QAction *m_projectCloseWithoutDocumentsAction;
 
     class FixedView
     {
