@@ -42,15 +42,19 @@
 #include <KAboutData>
 #include <KActionCollection>
 #include <KConfigGroup>
+#include <KLineEdit>
 #include <KPluginFactory>
 #include <KSharedConfig>
 #include <KToggleAction>
 #include <KXMLGUIFactory>
 #include <QAction>
+#include <QLineEdit>
 
 #include <ktexteditor/configinterface.h>
 #include <ktexteditor/cursor.h>
 
+#include <kfts_fuzzy_match.h>
+
 #include <QGroupBox>
 #include <QVBoxLayout>
 
@@ -110,12 +114,14 @@ KatePluginSymbolViewerView::KatePluginSymbolViewerView(KatePluginSymbolViewer *p
     m_currItemTimer.setSingleShot(true);
     connect(&m_currItemTimer, &QTimer::timeout, this, &KatePluginSymbolViewerView::updateCurrTreeItem);
 
-    QPixmap cls(class_xpm);
-
-    m_toolview = m_mainWindow->createToolView(plugin, QStringLiteral("kate_plugin_symbolviewer"), KTextEditor::MainWindow::Left, cls, i18n("Symbol List"));
+    m_toolview = m_mainWindow->createToolView(plugin,
+                                              QStringLiteral("kate_plugin_symbolviewer"),
+                                              KTextEditor::MainWindow::Left,
+                                              QIcon::fromTheme(QStringLiteral("class")),
+                                              i18n("Symbol List"));
 
     QWidget *container = new QWidget(m_toolview);
-    QHBoxLayout *layout = new QHBoxLayout(container);
+    QVBoxLayout *layout = new QVBoxLayout(container);
 
     m_symbols = new QTreeWidget();
     m_symbols->setFocusPolicy(Qt::NoFocus);
@@ -128,6 +134,14 @@ KatePluginSymbolViewerView::KatePluginSymbolViewerView(KatePluginSymbolViewer *p
     connect(m_symbols, &QTreeWidget::itemExpanded, this, &KatePluginSymbolViewerView::updateCurrTreeItem);
     connect(m_symbols, &QTreeWidget::itemCollapsed, this, &KatePluginSymbolViewerView::updateCurrTreeItem);
 
+    m_filter = new KLineEdit(container);
+    m_filter->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));
+    m_filter->setPlaceholderText(i18n("Filter..."));
+    m_filter->setClearButtonEnabled(true);
+    layout->addWidget(m_filter);
+
+    connect(m_filter, &KLineEdit::textChanged, this, &KatePluginSymbolViewerView::slotFilterChange);
+
     connect(m_mainWindow, &KTextEditor::MainWindow::viewChanged, this, &KatePluginSymbolViewerView::slotDocChanged);
 
     QStringList titles;
@@ -270,6 +284,9 @@ bool KatePluginSymbolViewerView::eventFilter(QObject *obj, QEvent *event)
             event->accept();
             return true;
         }
+    } else if (event->type() == QEvent::Show) {
+        slotDocChanged();
+        return true;
     }
     return QObject::eventFilter(obj, event);
 }
@@ -358,6 +375,8 @@ void KatePluginSymbolViewerView::parseSymbols()
         m_symbols->setSortingEnabled(true);
         m_symbols->sortItems(0, sortOrder);
     }
+
+    slotFilterChange(m_filter->text());
 }
 
 void KatePluginSymbolViewerView::goToSymbol(QTreeWidgetItem *it)
@@ -377,6 +396,31 @@ void KatePluginSymbolViewerView::goToSymbol(QTreeWidgetItem *it)
     kv->setCursorPosition(KTextEditor::Cursor(it->text(1).toInt(nullptr, 10), 0));
 }
 
+void KatePluginSymbolViewerView::slotFilterChange(const QString &text)
+{
+    QString filter = text.trimmed();
+    for (int i = 0; i < m_symbols->invisibleRootItem()->childCount(); ++i) {
+        QTreeWidgetItem *group_item = m_symbols->invisibleRootItem()->child(i);
+        for (int j = 0; j < group_item->childCount(); ++j) {
+            filterSymbols(group_item->child(j), filter);
+        }
+    }
+}
+
+bool KatePluginSymbolViewerView::filterSymbols(QTreeWidgetItem *item, const QString &filter)
+{
+    bool at_least_one_child_shown = false;
+    for (int i = 0; i < item->childCount(); ++i) {
+        if (filterSymbols(item->child(i), filter)) {
+            at_least_one_child_shown = true;
+        }
+    }
+    bool is_item_match = kfts::fuzzy_match_simple(filter, item->text(0));
+    bool is_item_shown = at_least_one_child_shown || filter.isEmpty() || is_item_match;
+    item->setHidden(!is_item_shown);
+    return is_item_shown;
+}
+
 KatePluginSymbolViewer::KatePluginSymbolViewer(QObject *parent, const QList<QVariant> &)
     : KTextEditor::Plugin(parent)
 {
@@ -468,13 +512,15 @@ QString KatePluginSymbolViewerConfigPage::name() const
 {
     return i18n("Symbol Viewer");
 }
+
 QString KatePluginSymbolViewerConfigPage::fullName() const
 {
     return i18n("Symbol Viewer Configuration Page");
 }
+
 QIcon KatePluginSymbolViewerConfigPage::icon() const
 {
-    return QPixmap(class_xpm);
+    return QIcon::fromTheme(QLatin1String("code-class"));
 }
 
 void KatePluginSymbolViewerConfigPage::apply()
@@ -484,5 +530,3 @@ void KatePluginSymbolViewerConfigPage::apply()
 // END KatePluginSymbolViewerConfigPage
 
 #include "plugin_katesymbolviewer.moc"
-
-// kate: space-indent on; indent-width 2; replace-tabs on;
