@@ -18,20 +18,17 @@ void KatePluginSymbolViewerView::parsePythonSymbols(void)
         return;
     }
 
-    m_macro->setText(i18n("Show Globals"));
+    m_macro->setText(i18n("Show Functions"));
     m_struct->setText(i18n("Show Methods"));
     m_func->setText(i18n("Show Classes"));
 
-    QString cl; // Current Line
-    QPixmap cls(class_xpm);
-    QPixmap mtd(method_xpm);
-    QPixmap mcr(macro_xpm);
-
-    int in_class = 0, state = 0, j;
+    Symbol type;
     QString name;
+    QString params;
+    QString current_class_name;
 
     QTreeWidgetItem *node = nullptr;
-    QTreeWidgetItem *mcrNode = nullptr, *mtdNode = nullptr, *clsNode = nullptr;
+    QTreeWidgetItem *functionNode = nullptr, *mtdNode = nullptr, *clsNode = nullptr;
     QTreeWidgetItem *lastMcrNode = nullptr, *lastMtdNode = nullptr, *lastClsNode = nullptr;
 
     KTextEditor::Document *kv = m_mainWindow->activeView()->document();
@@ -39,16 +36,16 @@ void KatePluginSymbolViewerView::parsePythonSymbols(void)
     // kdDebug(13000)<<"Lines counted :"<<kv->numLines()<<endl;
     if (m_treeOn->isChecked()) {
         clsNode = new QTreeWidgetItem(m_symbols, QStringList(i18n("Classes")));
-        mcrNode = new QTreeWidgetItem(m_symbols, QStringList(i18n("Globals")));
-        mcrNode->setIcon(0, QIcon(mcr));
-        clsNode->setIcon(0, QIcon(cls));
+        functionNode = new QTreeWidgetItem(m_symbols, QStringList(i18n("Functions")));
+        functionNode->setIcon(0, m_icon_function);
+        clsNode->setIcon(0, m_icon_class);
 
         if (m_expandOn->isChecked()) {
-            m_symbols->expandItem(mcrNode);
+            m_symbols->expandItem(functionNode);
             m_symbols->expandItem(clsNode);
         }
         lastClsNode = clsNode;
-        lastMcrNode = mcrNode;
+        lastMcrNode = functionNode;
         mtdNode = clsNode;
         lastMtdNode = clsNode;
         m_symbols->setRootIsDecorated(1);
@@ -56,9 +53,12 @@ void KatePluginSymbolViewerView::parsePythonSymbols(void)
         m_symbols->setRootIsDecorated(0);
     }
 
+    static const QRegularExpression class_regexp(QLatin1String("^class ([a-zA-Z0-9_]+)(\\((.*)\\))?:"));
+    static const QRegularExpression function_regexp(QLatin1String("^( *)def ([a-zA-Z_0-9]+)(\\(.*\\))?:"));
+    QRegularExpressionMatch match;
     for (int i = 0; i < kv->lines(); i++) {
         int line = i;
-        cl = kv->line(i);
+        QString cl = kv->line(i);
         // concatenate continued lines and remove continuation marker
         if (cl.length() == 0) {
             continue;
@@ -73,54 +73,36 @@ void KatePluginSymbolViewerView::parsePythonSymbols(void)
             }
         }
 
-        if (cl.indexOf(QRegularExpression(QLatin1String("^class [a-zA-Z0-9_,\\s\\(\\).]+:"))) >= 0) {
-            in_class = 1;
-        }
-
-        // if(cl.find( QRegularExpression(QLatin1String("[\\s]+def [a-zA-Z_]+[^#]*:")) ) >= 0) in_class = 2;
-        if (cl.indexOf(QRegularExpression(QLatin1String("^def\\s+[a-zA-Z_]+[^#]*:"))) >= 0) {
-            in_class = 0;
-        }
-
-        if (cl.indexOf(QLatin1String("def ")) >= 0 || (cl.indexOf(QLatin1String("class ")) >= 0 && in_class == 1)) {
-            if (cl.indexOf(QLatin1String("def ")) >= 0 && in_class == 1) {
-                in_class = 2;
-            }
-            state = 1;
-            if (cl.indexOf(QLatin1Char(':')) >= 0) {
-                state = 3; // found in the same line. Done
-            } else if (cl.indexOf(QLatin1Char('(')) >= 0) {
-                state = 2;
-            }
-
-            if (state == 2 || state == 3) {
-                name = cl.left(cl.indexOf(QLatin1Char('(')));
-            }
-        }
-
-        if (state > 0 && state < 3) {
-            for (j = 0; j < cl.length(); j++) {
-                if (cl.at(j) == QLatin1Char('(')) {
-                    state = 2;
-                } else if (cl.at(j) == QLatin1Char(':')) {
-                    state = 3;
-                    break;
-                }
-
-                if (state == 1) {
-                    name += cl.at(j);
+        match = class_regexp.match(cl);
+        if (match.hasMatch()) {
+            type = Symbol::Class;
+        } else {
+            match = function_regexp.match(cl);
+            if (match.hasMatch()) {
+                if (match.captured(1).isEmpty() || current_class_name.isEmpty() // case where function is declared inside a block
+                ) {
+                    type = Symbol::Function;
+                    current_class_name.clear();
+                } else {
+                    type = Symbol::Method;
                 }
             }
         }
-        if (state == 3) {
-            // qDebug(13000)<<"Function -- Inserted : "<<name<<" at row : "<<i;
-            if (in_class == 1) { // strip off the word "class "
-                name = name.trimmed().mid(6);
-            } else { // strip off the word "def "
-                name = name.trimmed().mid(4);
+
+        if (match.hasMatch()) {
+            if (type == Symbol::Class) {
+                name = match.captured(1);
+                params = match.captured(2);
+                current_class_name = name;
+            } else {
+                name = match.captured(2);
+                params = match.captured(3);
+            }
+            if (m_typesOn->isChecked()) {
+                name += params;
             }
 
-            if (m_func->isChecked() && in_class == 1) {
+            if (m_func->isChecked() && type == Symbol::Class) {
                 if (m_treeOn->isChecked()) {
                     node = new QTreeWidgetItem(clsNode, lastClsNode);
                     if (m_expandOn->isChecked()) {
@@ -134,11 +116,11 @@ void KatePluginSymbolViewerView::parsePythonSymbols(void)
                 }
 
                 node->setText(0, name);
-                node->setIcon(0, QIcon(cls));
+                node->setIcon(0, m_icon_class);
                 node->setText(1, QString::number(line, 10));
             }
 
-            if (m_struct->isChecked() && in_class == 2) {
+            if (m_struct->isChecked() && type == Symbol::Method) {
                 if (m_treeOn->isChecked()) {
                     node = new QTreeWidgetItem(mtdNode, lastMtdNode);
                     lastMtdNode = node;
@@ -147,25 +129,25 @@ void KatePluginSymbolViewerView::parsePythonSymbols(void)
                 }
 
                 node->setText(0, name);
-                node->setIcon(0, QIcon(mtd));
+                node->setIcon(0, m_icon_function);
                 node->setText(1, QString::number(line, 10));
             }
 
-            if (m_macro->isChecked() && in_class == 0) {
+            if (m_macro->isChecked() && type == Symbol::Function) {
                 if (m_treeOn->isChecked()) {
-                    node = new QTreeWidgetItem(mcrNode, lastMcrNode);
+                    node = new QTreeWidgetItem(functionNode, lastMcrNode);
                     lastMcrNode = node;
                 } else {
                     node = new QTreeWidgetItem(m_symbols);
                 }
 
                 node->setText(0, name);
-                node->setIcon(0, QIcon(mcr));
+                node->setIcon(0, m_icon_function);
                 node->setText(1, QString::number(line, 10));
             }
 
-            state = 0;
             name.clear();
+            params.clear();
         }
     }
 }
